#!/usr/bin/env python
"""\
Rename files using regular expression substitutions.

regren [option] [<pattern> <substitute>] <glob> [<glob>...]

Options may be:
    -t      - TEST MODE: no files will be renamed (recommended!)
    -r      - replace mode: the subsitute pattern replaces the entire
              original filename rather than just substituting the
              regular expression matched area.
    -s      - use special codes (listed below)
    -d      - debug mode (print extra info about the process)
    -m      - convert all files to mixed case (no pattern/sub needed)
    -l      - convert all files to lower case (no pattern/sub needed)
    -u      - convert all files to upper case (no pattern/sub needed)

Special Codes - these codes using -s have special meanings in <subsitute>:
    %(fn)s  - substitutes the whole original filename
    %(count)d - subsitutes the counter value (increments for each file)
    %(isodate)s - inserts the date in ISO order (digits only: 20001130)
    %(isotime)s - inserts the time in ISO order (digits only: 230741)

Notes:
    * you can use brackets in the <pattern> to 'capture' parts of or whole
      pattersn and then use \1 .. \9 escape codes in the substution string
      to re-insert those patterns.
    * you can specify leading zeros in the %(count)d special code like
      this (to have leading zeros pad up to 4 spaces): %(count)04d
    * matching is insensitive (i.e. the pattern)
    * to use special codes in DOS you may have to use %% or even %%%%
      instead of a single percent sign (use test mode to see results)
    * it is strongly recommended that you always use "test" mode first!
    * be careful replacing "." ! You could replace every character!
    * capitalize mode trims and compresses runs of white space and

$Id: regren.py,v 1.4 2001/01/10 17:32:28 x Exp $
"""

import sys,os,glob,re,string,time

def help():
    print globals()['__doc__']
    sys.exit()

count = 0
testmode = None
replacemode = None
casemode = None
specials = None
debugmode = None

isotime = time.strftime("%H%M%S",time.localtime(time.time()))
isodate = time.strftime("%Y%m%d",time.localtime(time.time()))

# parse arguments
while sys.argv[1][0] == "-" and len(sys.argv[1])>1 and \
        sys.argv[1][1] in ('t','r','s','d','h','l','m','u'):
    if sys.argv[1][1] == 't':
        testmode = 1
    elif sys.argv[1][1] == 'r':
        replacemode = 1
    elif sys.argv[1][1] == 's':
        specials = 1
    elif sys.argv[1][1] == 'l':
        casemode = 1
    elif sys.argv[1][1] == 'u':
        casemode = 2
    elif sys.argv[1][1] == 'm':
        casemode = 3
    elif sys.argv[1][1] == 'd':
        debugmode = 1
    elif sys.argv[1][1] == 'h':
        help()
    del sys.argv[1]

if debugmode:
    print "Args (%s): `%s`" % (len(sys.argv)-1,string.join(sys.argv[1:],"`,`"))

if casemode:
    if len(sys.argv)<=1:
        help()
else:
    if len(sys.argv)<=3:
        help()
    pattern = sys.argv[1]
    subst   = sys.argv[2]
    del sys.argv[1]
    del sys.argv[1]

# build file list
fns = []
for fn in sys.argv[1:]:
    fns.extend(glob.glob(fn))

if debugmode:
    if casemode:
        print "Casemode:",
        if casemode == 1:
            print "lower"
        elif casemode == 2:
            print "upper"
        elif casemode == 3:
            print "capitalize"
        else:
            print "unknown"
    else:
        print "Pattern: %s" % pattern
        print "Subst  : %s" % subst
    print "# Files: %s" % len(fns)
    if testmode: print "Testmode: On"
    if specials: print "Specials: On (see help file)"

if fns:
    if not casemode:
        find = re.compile(pattern,re.I)
    for fn in fns:
        if replacemode:
            newfn = subst
            x = find.findall(fn)
            if debugmode:
                print "found: %s" % x
            if x:
                x = (None,) + x[0]
                for i in range(1,len(x)):
                    newfn = string.replace(newfn,r"\%s" % i, x[i])
        elif casemode:
            if casemode == 1:
                newfn = string.lower(fn)
            elif casemode == 2:
                newfn = string.upper(fn)
            elif casemode == 3:
                newfn = string.capwords(fn)
            else:
                newfn = fn
        else:
            newfn = find.sub(subst,fn)
        if specials and (string.find(newfn,"%")>-1):
            newfn = newfn % {'count': count, 'fn': fn,
                    'isotime': isotime, 'isodate': isodate}
        if newfn <> fn:
            try:
                if testmode:
                    print "TEST:",
                else:
                    os.rename(fn,newfn)
                print "%s -> %s" % (fn,newfn)
                count = count + 1
            except:
                print "Error: Could not rename '%s' to '%s'" % (fn,newfn)
        elif testmode or debugmode:
            print "TEST: %s == %s" % (fn,newfn)


print "%s out of %s file(s) renamed." % (count,len(fns))

